# -*- coding: utf-8 -*-
from django.conf.urls import url
from assessment.views.api import result


urlpatterns = [
    url(
        regex=r'^$',
        view=result.list_results,
        name='list_results',
    ),
    url(
        regex=r'^create/$',
        view=result.create_result,
        name='create_result',
    ),
    url(
        regex=r'^update/(?P<uuid>[-\w]+)/$',
        view=result.update_result,
        name='update_result',
    ),
    url(
        regex=r'^(?P<uuid>[-\w]+)/$',
        view=result.retrieve_result,
        name='retrieve_result',
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-08-11 19:22
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('assessment', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='answer',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='assessment.Question', verbose_name='question'),
        ),
        migrations.AlterField(
            model_name='answer',
            name='result',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='assessment.Result', verbose_name='result'),
        ),
        migrations.AlterField(
            model_name='question',
            name='survey',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='assessment.Survey', verbose_name='survey'),
        ),
        migrations.AlterField(
            model_name='result',
            name='survey',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='results', to='assessment.Survey', verbose_name='survey'),
        ),
        migrations.AlterField(
            model_name='result',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='results', to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
    ]



# -*- coding: utf-8 -*-
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
# from assessment.serializers import ChoiceSerializer
# from assessment.models import Choice


@api_view(['GET', ])
def retrieve_choice(request, uuid):
    """
    """
    if request.method == 'GET':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['POST', ])
def create_choice(request):
    """
    """
    if request.method == 'POST':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['PUT', ])
def update_choice(request, uuid):
    """
    """
    if request.method == 'PUT':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['GET', ])
def list_choices(request):
    """
    """
    if request.method == 'GET':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)



"""API urls for the django-assessment project"""
from django.conf.urls import url
from django.conf.urls import include


urlpatterns = [
    url(r'^answer/', include('assessment.urls.api.answer')),
    url(r'^choice/', include('assessment.urls.api.choice')),
    url(r'^question/', include('assessment.urls.api.question')),
    url(r'^result/', include('assessment.urls.api.result')),
    url(r'^survey/', include('assessment.urls.api.survey')),
]



# -*- coding: utf-8 -*-
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from assessment.serializers import ResultSerializer
from assessment.models import Result


@api_view(['GET', ])
def retrieve_result(request, uuid):
    if request.user.is_authenticated:
        try:
            result = Result.objects.get(pk=uuid)
        except Result.DoesNotExist:
            return Response({"error": "result not found"}, status.HTTP_404_NOT_FOUND)

        if request.user == result.user or request.user == result.survey.admin:
            serializer = ResultSerializer(result)
            return Response(serializer.data)
        else:
            return Response({"error": "un-authorized"}, status=status.HTTP_403_FORBIDDEN)
    else:
        return Response({"error": "please log in"}, status.HTTP_401_UNAUTHORIZED)


@api_view(['POST', ])
def create_result(request):
    """
    API endpoint to respond to a survery.
    """
    if request.user.is_authenticated:
        request.data['user'] = request.user.pk
        serializer = ResultSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    else:
        return Response({"error": "please log in"}, status.HTTP_401_UNAUTHORIZED)


@api_view(['PUT', ])
def update_result(request, uuid):
    """
    """
    if request.method == 'PUT':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['GET', ])
def list_results(request):
    """
    Generate a list of all survey results for the authenticated user.
    If there is no authenticated user, return 401 UNAUTHORIZED.
    """
    if request.user.is_authenticated():
        results = Result.objects.filter(user=request.user)
        serializer = ResultSerializer(results, many=True)
        return Response(serializer.data, status.HTTP_200_OK)
    return Response({"details": "please log in"}, status.HTTP_401_UNAUTHORIZED)



# -*- coding: utf-8 -*-
from django.db.models import Q
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from assessment.serializers import SurveySerializer
from assessment.models import Survey


@api_view(['POST', ])
def create_survey(request):
    if request.user.is_authenticated:
        serializer = SurveySerializer(data=request.data)
        if serializer.is_valid():
            serializer.save(admin=request.user)
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    else:
        return Response({"error": "please login"}, status.HTTP_401_UNAUTHORIZED)


@api_view(['PUT', ])
def update_survey(request, uuid):
    """
    """
    if request.user.is_authenticated:
        try:
            survey = Survey.objects.get(pk=uuid)
        except Survey.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)

        if request.user == survey.admin:
            serializer = SurveySerializer(survey, data=request.data)
            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data, status=status.HTTP_200_OK)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response({"error": "un-authorized"}, status=status.HTTP_403_FORBIDDEN)
    else:
        return Response({"error": "user not found"}, status=status.HTTP_404_NOT_FOUND)


@api_view(['GET', ])
def retrieve_survey(request, uuid):
    """
    return a single survey.
    """
    try:
        result = Survey.objects.get(pk=uuid)
    except (Survey.DoesNotExist, ValueError):
        return Response({"error": "Survey not found"}, status=status.HTTP_404_NOT_FOUND)

    if result.is_private:
        if request.user.is_authenticated:
            if request.user == result.admin or request.user in result.users.all():
                serializer = SurveySerializer(result, context={'request': request})
                return Response(serializer.data)
            else:
                return Response({"error": "This is a private survey."}, status=status.HTTP_403_FORBIDDEN)
        else:
            return Response({"error": "Please login."}, status=status.HTTP_401_UNAUTHORIZED)
    else:
        serializer = SurveySerializer(result, context={'request': request})
        return Response(serializer.data)


@api_view(['GET', ])
def list_surveys(request):
    """
    return a list of surveys.
    """
    if request.user.is_authenticated:
        public_surveys = Q(is_private=False)
        private_surveys = Q(is_private=True, pk__in=request.user.assessment_user_surveys.all())
        admin_surveys = Q(admin=request.user)
        surveys = Survey.objects.filter(public_surveys | private_surveys | admin_surveys)
    else:
        surveys = Survey.objects.filter(is_private=False)
    serializer = SurveySerializer(surveys, many=True, context={'request': request})
    return Response(serializer.data, status=status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from django.conf.urls import url
from assessment.views.api import question


urlpatterns = [
    url(
        regex=r'^$',
        view=question.list_questions,
        name='list_questions',
    ),
    url(
        regex=r'^create/$',
        view=question.create_question,
        name='create_question',
    ),
    url(
        regex=r'^update/(?P<uuid>[-\w]+)/$',
        view=question.update_question,
        name='update_question',
    ),
    url(
        regex=r'^(?P<uuid>[-\w]+)/$',
        view=question.retrieve_question,
        name='retrieve_question',
    ),
]



"""Defaults urls for the django-assessment project"""
from django.conf.urls import url
from django.conf.urls import include


urlpatterns = [
    url(r'^api/', include('assessment.urls.api', namespace='api')),
]



# -*- coding: utf-8 -*-
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from rest_framework import status
from rest_framework.test import APITestCase
from assessment.models import Survey



class ResultTests(APITestCase):

    def setUp(self):
        try:
            self.bob = User.objects.create_superuser(
                username='bob',
                email='bob@example.com',
                password='top_secret'
            )
        except Exception:
            self.bob = User.objects.get(username='bob')
        try:
            self.admin = User.objects.create_superuser(
                username='admin',
                email='admin@example.com',
                password='top_secret'
            )
        except Exception:
            self.admin = User.objects.get(username='admin')
        self.client.login(username='bob', password='top_secret')

    def tearDown(self):
        Survey.objects.all().delete()

    def test_getlist_unauthenticated(self):
        self.client.logout()
        response = self.client.get(reverse('assessment:api:list_results'))
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_create_response_unauthenticated(self):
        self.client.logout()
        response = self.client.get(reverse('assessment:api:list_results'))
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_getlist_authenticated(self):
        self.client.login(username='bob', password='top_secret')
        response = self.client.get(reverse('assessment:api:list_results'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from django.conf.urls import url
from assessment.views.api import survey


urlpatterns = [
    url(
        regex=r'^$',
        view=survey.list_surveys,
        name='list_surveys',
    ),
    url(
        regex=r'^create/$',
        view=survey.create_survey,
        name='create_survey',
    ),
    url(
        regex=r'^update/(?P<uuid>[-\w]+)/$',
        view=survey.update_survey,
        name='update_survey',
    ),
    url(
        regex=r'^(?P<uuid>[-\w]+)/$',
        view=survey.retrieve_survey,
        name='retrieve_survey',
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-09-07 14:58
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('assessment', '0002_auto_20170811_1922'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='surveytranslation',
            name='slug',
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from rest_framework import status
from rest_framework.test import APITestCase
from assessment.models import Survey


class SurveyTests(APITestCase):

    def setUp(self):
        try:
            self.admin = User.objects.create_superuser(
                username='admin',
                email='admin@example.com',
                password='top_secret'
            )
        except Exception:
            self.admin = User.objects.get(username='admin')
        self.client.login(username='admin', password='top_secret')

    def tearDown(self):
        Survey.objects.all().delete()

    def test_create_public_survey(self):
        """
        Ensure we can create a new survey object.
        """
        url = reverse('assessment:api:create_survey')
        data = {
            "translations": {
                "de": {
                    "name": "Hallo Welt",
                    "slug": "hallo-welt",
                    "description": "Wie sage ich; 'Hello World', auf Deutsch?",
                },
                "en": {
                    "name": "Hello World",
                    "slug": "hello-world",
                    "description": "How do I say; 'Hello world', in English?",
                },
                "nl": {
                    "name": "Hallo Wereld",
                    "slug": "hallo-wereld",
                    "description": "Hoe zeg ik; 'Hello World', in Nederlandse?",
                }
            },
            "is_private": False,
            "is_active": True,
            "start_date_time": datetime.datetime.now(),
        }
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Survey.objects.get().name, 'Hello World')

    def test_create_private_survey(self):
        """
        Ensure we can create a new survey object.
        """
        url = reverse('assessment:api:create_survey')
        data = {
            "translations": {
                "de": {
                    "name": "Hallo Welt",
                    "slug": "hallo-welt",
                    "description": "Wie sage ich; 'Hello World', auf Deutsch?",
                },
                "en": {
                    "name": "Hello World",
                    "slug": "hello-world",
                    "description": "How do I say; 'Hello world', in English?",
                },
                "nl": {
                    "name": "Hallo Wereld",
                    "slug": "hallo-wereld",
                    "description": "Hoe zeg ik; 'Hello World', in Nederlandse?",
                }
            },
            "is_private": True,
            "is_active": True,
            "start_date_time": datetime.datetime.now(),
        }
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Survey.objects.get().name, 'Hello World')

    def test_getlist(self):
        response = self.client.get(reverse('assessment:api:list_surveys'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from rest_framework import status
from rest_framework.test import APITestCase


class AnswerTests(APITestCase):
    def test_getlist(self):
        response = self.client.get(reverse('assessment:api:list_answers'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from assessment.serializers import AnswerSerializer


@api_view(['GET', ])
def retrieve_answer(request, uuid):
    """
    """
    if request.method == 'GET':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['POST', ])
def create_answer(request):
    """
    """
    if request.user.is_authenticated:
        serializer = AnswerSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            try:
                serializer.save()
            except Exception as error:
                print(error)
                return Response("Valid but failed to save", status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            else:
                return Response(serializer.data, status=status.HTTP_201_CREATED)
        else:
            return Response(serializer.errors, status=status.HTTP_406_NOT_ACCEPTABLE)
    else:
        return Response({"error": "please login"}, status.HTTP_401_UNAUTHORIZED)


@api_view(['PUT', ])
def update_answer(request, uuid):
    """
    """
    if request.method == 'PUT':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['GET', ])
def list_answers(request):
    """
    """
    if request.method == 'GET':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['GET', ])
def filter_answers(request):
    """
    """
    if request.method == 'GET':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from django.conf.urls import url
from assessment.views.api import choice


urlpatterns = [
    url(
        regex=r'^$',
        view=choice.list_choices,
        name='list_choices',
    ),
    url(
        regex=r'^create/$',
        view=choice.create_choice,
        name='create_choice',
    ),
    url(
        regex=r'^update/(?P<uuid>[-\w]+)/$',
        view=choice.update_choice,
        name='update_choice',
    ),
    url(
        regex=r'^(?P<uuid>[-\w]+)/$',
        view=choice.retrieve_choice,
        name='retrieve_choice',
    ),
]



# -*- coding: utf-8 -*-
import os
import uuid
import environ

env = environ.Env()

gettext = lambda s: s  # noqa

DEBUG = env.bool('DJANGO_DEBUG', default=False)

ALLOWED_HOSTS = env.list('ALLOWED_HOSTS', default="*")

DATABASES = {
    'default': env.db('DATABASE_URL', default='sqlite:////tmp/assessment.db'),
}

TIME_ZONE = 'UTC'

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(os.path.dirname(__file__), 'static')
MEDIA_URL = '/media/'

SECRET_KEY = str(uuid.uuid4())
SITE_ID = 1

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'demo.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'rest_framework',
    'rest_framework.authtoken',
    'corsheaders',
    'parler',
    'allauth',
    'allauth.account',
    'rest_auth',
    'rest_auth.registration',
    'assessment'
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

USE_TZ = True
USE_I18N = True
USE_L10N = True

LANGUAGE_CODE = 'en'

LANGUAGES = [
    ('de', gettext('German')),
    ('en', gettext('English')),
    ('fr', gettext('French')),
    ('nl', gettext('Nederlands')),
]

# ------------------------------------------------------------------------------
# TRANSLATION Configuration
# ------------------------------------------------------------------------------
PARLER_DEFAULT_LANGUAGE_CODE = LANGUAGE_CODE
PARLER_LANGUAGES = {
    None: (
        {'code': 'de',},
        {'code': 'en',},
        {'code': 'fr',},
        {'code': 'nl',},
    ),
    'default': {
        'fallbacks': [LANGUAGE_CODE],
        'hide_untranslated': False,
    }
}

STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'

CORS_ORIGIN_WHITELIST = (
    'vandorjw.github.io',
    'localhost:8080',
)

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.TokenAuthentication',
    )
}

ACCOUNT_EMAIL_VERIFICATION = False

EMAIL_USE_TLS = True
EMAIL_HOST = env('EMAIL_HOST', default='localhost')
EMAIL_HOST_USER = env('EMAIL_HOST_USER', default='')
EMAIL_HOST_PASSWORD = env('EMAIL_HOST_PASSWORD', default='')
EMAIL_PORT = 587



# -*- coding: utf-8 -*-
from rest_framework import serializers
from parler_rest.serializers import (
    TranslatableModelSerializer,
    TranslatedFieldsField,
)
from assessment.models import (
    Survey,
    Question,
    Choice,
    Result,
    Answer,
)


class QuestionSerializer(TranslatableModelSerializer):
    _uid = serializers.UUIDField(label='ID', read_only=True)
    translations = TranslatedFieldsField(shared_model=Question)
    url = serializers.HyperlinkedIdentityField(
        read_only=True,
        view_name='assessment:api:retrieve_question',
        lookup_field='pk',
        lookup_url_kwarg='uuid',
    )

    class Meta:
        model = Question
        fields = (
            '_uid',
            'survey',
            'of_type',
            'translations',
            'url',
        )


class SurveySerializer(TranslatableModelSerializer):
    _uid = serializers.UUIDField(label='ID', read_only=True)
    translations = TranslatedFieldsField(shared_model=Survey, write_only=True)
    questions = QuestionSerializer(many=True, read_only=True)
    is_admin = serializers.SerializerMethodField()
    in_users = serializers.SerializerMethodField()
    name = serializers.SerializerMethodField()
    description = serializers.SerializerMethodField()

    class Meta:
        model = Survey
        fields = (
            '_uid',
            'is_active',
            'is_private',
            'start_date_time',
            'end_date_time',
            'translations',
            'questions',
            'is_admin',
            'in_users',
            'name',
            'description',

        )

    def get_is_admin(self, obj):
        """
        The application should never directly report who the admin is.
        This method reports if the current authenticated user is the admin.
        """
        try:
            user = self.context.get('request').user
        except Exception:
            # raise serializers.ValidationError('Could not access request.user')
            return False
        if user == obj.admin:
            return True
        else:
            return False

    def get_in_users(self, obj):
        """
        The application should never directly report the entire user group.
        This method reports if the current authenticated user is users.
        """
        try:
            user = self.context.get('request').user
        except Exception:
            # raise serializers.ValidationError('Could not access request.user')
            return False
        if user in obj.users.all():
            return True
        else:
            return False

    def get_name(self, obj):
        return obj.name

    def get_description(self, obj):
        return obj.description


class ChoiceSerializer(TranslatableModelSerializer):
    _uid = serializers.UUIDField(label='ID', read_only=True)
    translations = TranslatedFieldsField(shared_model=Choice)

    class Meta:
        model = Choice
        fields = (
            '_uid',
            'question',
            'is_correct',
            'translations',
        )


class ResultSerializer(serializers.ModelSerializer):
    _uid = serializers.UUIDField(label='ID', read_only=True)

    class Meta:
        model = Result
        fields = (
            '_uid',
            'survey',
            'user',
        )

    def validate(self, attrs):
        instance = Result(**attrs)
        instance.clean()
        return attrs


class AnswerSerializer(serializers.ModelSerializer):
    _uid = serializers.UUIDField(label='ID', read_only=True)

    class Meta:
        model = Answer
        fields = (
            '_uid',
            'result',
            'question',
            'answer',
        )

    def validate(self, attrs):
        instance = Answer(**attrs)
        instance.clean()
        return attrs

    def validate_result(self, result):
        try:
            user = self.context.get('request').user
        except Exception:
            raise serializers.ValidationError('Could not access request.user')

        if not result.user == user:
            raise serializers.ValidationError('result does not belong to current user!')
        else:
            return result
